"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.KeysUtils = void 0;
const lodash_1 = require("lodash");
const models_1 = require("./../models");
class KeysUtils {
    static groupKeysByName(keys) {
        return lodash_1.chain(keys).groupBy("name").map((dictionary, key) => {
            const views = lodash_1.union(...lodash_1.map(dictionary, 'views'));
            const languages = lodash_1.union(...lodash_1.map(dictionary, 'languages'));
            const item = new models_1.KeyModel(key, views, languages);
            return item;
        }).value();
    }
    static findKeysList(keys, customRegExp = []) {
        const keysFromDirectiveInsideTag = `(?<=<[^<]*[\\s*\\w*\\s*]*(translate|TRANSLATE)(?!\\}|\\s{0,}\\")[\\s*\\w*\\s*]*[^>]*>\\s*)([a-zA-Z0-9_.]*)(?=\\s*<\\s*\\/.*\\s*>)`;
        const keysFromDirectiveInView = `(?<=translate=["']{1,2}|\\[translate\\]=["']{1,2})([A-Za-z0-9_.]+)(?=["']{1,2})`;
        const keysFromPipeInView = `(?<=['"])([a-zA-Z0-9_\\-.]*)(?=['"]\\s?\\|\\s?translate|['"](\\s*\\|\\s*\\w*)*translate)`;
        const keysListRegExp = keys.map((key) => {
            const regExpForSingleKey = `(?<=[^\\w.-])${key.replace('.', '\\.')}(?=[^\\w.-])`;
            return regExpForSingleKey;
        }).join('|');
        // @ts-ignore
        const customRegExpList = customRegExp.map((regexp) => {
            return `(${regexp})`;
        });
        const resultKeysRegExp = [
            keysListRegExp,
            keysFromPipeInView,
            keysFromDirectiveInView,
            keysFromDirectiveInsideTag,
            ...customRegExpList
        ];
        return new RegExp(resultKeysRegExp.join('|'), 'gm');
    }
}
exports.KeysUtils = KeysUtils;
//# sourceMappingURL=keys.js.map